const fs = require('fs-extra');
const axios = require('axios');
const path = require('path');
const ora = require('ora');
const run = require('@sbspk/run');
const sevenZip = require('7zip-bin').path7za;

const downloadUrl = 'https://github.com/oneclick/rubyinstaller2/releases/download/RubyInstaller-2.6.3-1/rubyinstaller-2.6.3-1-x86.7z';
const downloadDir = path.resolve(__dirname, '../temp');
const rubyDir = path.resolve(__dirname, '../ruby');

async function clean() {
  const spinner = ora('Cleaning ruby directory').start();
  await fs.remove(rubyDir);
  spinner.succeed();
}

async function download() {
  const spinner = ora('Downloading ruby').start();
  const response = await axios.get(downloadUrl, { responseType: 'arraybuffer' });
  await fs.outputFile(path.join(downloadDir, 'ruby.7z'), response.data);
  spinner.succeed();
}

async function extract() {
  const spinner = ora('Extracting ruby').start();
  await run(sevenZip, ['x', '-y', 'ruby.7z'], { cwd: downloadDir });
  const extractedDirName = path.basename(new URL(downloadUrl).pathname, '.7z');
  const extractedDir = path.resolve(downloadDir, extractedDirName);
  await fs.move(extractedDir, rubyDir, { overwrite: true });
  await fs.remove(downloadDir);
  spinner.succeed();
}

Promise.resolve()
  .then(clean)
  .then(download)
  .then(extract)
  .catch((err)=> {
    console.error(err);
    process.exit(1);
  });
