const ora = require('ora');
const path = require('path');
const fs = require('fs-extra');
const run = require('@sbspk/run');
const packageJson = require('../package.json');
const rubyDir = path.resolve(__dirname, '../ruby');
const gemDir = path.resolve(__dirname, '../ruby/gems');

async function clean() {
  const spinner = ora('Cleaning gem directory').start();
  await fs.remove(gemDir);
  await fs.ensureDir(gemDir);
  spinner.succeed();
}

async function installGem({ gem, version }) {
  const spinner = ora(`Installing ${gem} ${version}`).start();
  const args = ['install', gem, `--version=${version}`, '--no-document' ];
  let command = 'gem';

  // use local ruby and gem on Windows
  if(process.platform === 'win32') {
    args.unshift(path.join(rubyDir, 'bin/gem'));
    command = path.join(rubyDir, 'bin/ruby.exe');
  }

  // install Gem
  await run(command, args, {
    env: { ...process.env, GEM_HOME: gemDir, GEM_PATH: gemDir }
  });

  spinner.succeed();
}

async function installGems() {
  const gems = packageJson.gems || {};
  for(const [gem, version] of Object.entries(gems)) {
    await installGem({ gem, version });
  }
}

Promise.resolve()
  .then(clean)
  .then(installGems)
  .catch((err)=> {
    console.error(err);
    process.exit(1);
  });
